UI_TRANSLATIONS = {
    'button_NER_highlighting': {
        'ar': 'التوكيد',
        'en': 'Highlight',
        'da': 'Fremhævelse',
        'de': 'Hervorheben',
        'fr': 'Marquer',
        'es': 'Marca',
        'nl': 'Benadrukken'
    },
    'button_timestamps': {
        'ar': 'الطابع الزمني',
        'en': 'Timestamps',
        'da': 'Tidsstempel',
        'de': 'Zeitstempel',
        'fr': 'Horodatage',
        'es': 'Marca de tiempo',
        'nl': 'Tijdstempel'
    },
    'button_show_HTML': {
        'ar': 'إظهار HTML',
        'en': 'Show HTML',
        'da': 'Vis HTML',
        'de': 'HTML anzeigen',
        'fr': 'Afficher HTML',
        'es': 'Mostrar HTML',
        'nl': 'HTML weergeven'
    },
    'button_save': {
        'ar': 'الحفظ',
        'en': 'Save',
        'da': 'Gemme',
        'de': 'Speichern',
        'fr': 'Enregistrer',
        'es': 'Guardar',
        'nl': 'Sla'
    },
    'heading_topics': {
        'ar': 'المواضيع',
        'en': 'Topics',
        'da': 'Temaer',
        'de': 'Themen',
        'fr': 'Thèmes',
        'es': 'Temas',
        'nl': 'Onderwerpen'
    },
    'summary': {
        'ar': 'الملخص: ',
        'en': 'Summary: ',
        'da': 'Sammenfatning: ',
        'de': 'Zusammenfassung: ',
        'fr': 'Résumé: ',
        'es': 'Resumen: ',
        'nl': 'Samenvatting: '
    },
    'keywords': {
        'ar': 'الكلمات المفتاحية: ',
        'en': 'Keywords: ',
        'da': 'Nøgleord: ',
        'de': 'Schlagworte: ',
        'fr': 'Mots-clés: ',
        'es': 'Palabras clave: ',
        'nl': 'Trefwoorden: '
    }
}


LANGUAGES = {
    "en": "english",
    "zh": "chinese",
    "de": "german",
    "es": "spanish",
    "ru": "russian",
    "ko": "korean",
    "fr": "french",
    "ja": "japanese",
    "pt": "portuguese",
    "tr": "turkish",
    "pl": "polish",
    "ca": "catalan",
    "nl": "dutch",
    "ar": "arabic",
    "sv": "swedish",
    "it": "italian",
    "id": "indonesian",
    "hi": "hindi",
    "fi": "finnish",
    "vi": "vietnamese",
    "he": "hebrew",
    "uk": "ukrainian",
    "el": "greek",
    "ms": "malay",
    "cs": "czech",
    "ro": "romanian",
    "da": "danish",
    "hu": "hungarian",
    "ta": "tamil",
    "no": "norwegian",
    "th": "thai",
    "ur": "urdu",
    "hr": "croatian",
    "bg": "bulgarian",
    "lt": "lithuanian",
    "la": "latin",
    "mi": "maori",
    "ml": "malayalam",
    "cy": "welsh",
    "sk": "slovak",
    "te": "telugu",
    "fa": "persian",
    "lv": "latvian",
    "bn": "bengali",
    "sr": "serbian",
    "az": "azerbaijani",
    "sl": "slovenian",
    "kn": "kannada",
    "et": "estonian",
    "mk": "macedonian",
    "br": "breton",
    "eu": "basque",
    "is": "icelandic",
    "hy": "armenian",
    "ne": "nepali",
    "mn": "mongolian",
    "bs": "bosnian",
    "kk": "kazakh",
    "sq": "albanian",
    "sw": "swahili",
    "gl": "galician",
    "mr": "marathi",
    "pa": "punjabi",
    "si": "sinhala",
    "km": "khmer",
    "sn": "shona",
    "yo": "yoruba",
    "so": "somali",
    "af": "afrikaans",
    "oc": "occitan",
    "ka": "georgian",
    "be": "belarusian",
    "tg": "tajik",
    "sd": "sindhi",
    "gu": "gujarati",
    "am": "amharic",
    "yi": "yiddish",
    "lo": "lao",
    "uz": "uzbek",
    "fo": "faroese",
    "ht": "haitian creole",
    "ps": "pashto",
    "tk": "turkmen",
    "nn": "nynorsk",
    "mt": "maltese",
    "sa": "sanskrit",
    "lb": "luxembourgish",
    "my": "myanmar",
    "bo": "tibetan",
    "tl": "tagalog",
    "mg": "malagasy",
    "as": "assamese",
    "tt": "tatar",
    "haw": "hawaiian",
    "ln": "lingala",
    "ha": "hausa",
    "ba": "bashkir",
    "jw": "javanese",
    "su": "sundanese",
    "yue": "cantonese",
}
